"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaUtils = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const web3_js_1 = require("@solana/web3.js");
const constants_1 = require("./constants");
const platform_1 = require("./platform");
const address_1 = require("./address");
const spl_token_1 = require("@solana/spl-token");
// forces SolanaUtils to implement PlatformUtils
var _ = SolanaUtils;
/**
 * @category Solana
 */
// Provides runtime concrete value
var SolanaUtils;
(function (SolanaUtils) {
    function nativeTokenId(chain) {
        if (!isSupportedChain(chain))
            throw new Error(`invalid chain: ${chain}`);
        return {
            chain: chain,
            address: new address_1.SolanaAddress(address_1.SolanaZeroAddress), // TODO: fix weird type error
        };
    }
    SolanaUtils.nativeTokenId = nativeTokenId;
    function isSupportedChain(chain) {
        const platform = (0, connect_sdk_1.chainToPlatform)(chain);
        return platform === platform_1.SolanaPlatform.platform;
    }
    SolanaUtils.isSupportedChain = isSupportedChain;
    function isNativeTokenId(chain, tokenId) {
        if (!isSupportedChain(chain))
            return false;
        if (tokenId.chain !== chain)
            return false;
        const native = nativeTokenId(chain);
        return native == tokenId;
    }
    SolanaUtils.isNativeTokenId = isNativeTokenId;
    async function getDecimals(chain, rpc, token) {
        if (token === 'native')
            return (0, connect_sdk_1.nativeDecimals)(platform_1.SolanaPlatform.platform);
        let mint = await rpc.getParsedAccountInfo(new address_1.SolanaAddress(token).unwrap());
        if (!mint || !mint.value)
            throw new Error('could not fetch token details');
        const { decimals } = mint.value.data.parsed.info;
        return BigInt(decimals);
    }
    SolanaUtils.getDecimals = getDecimals;
    async function getBalance(chain, rpc, walletAddress, token) {
        if (token === 'native')
            return BigInt(await rpc.getBalance(new web3_js_1.PublicKey(walletAddress)));
        const splToken = await rpc.getTokenAccountsByOwner(new web3_js_1.PublicKey(walletAddress), { mint: new address_1.SolanaAddress(token).unwrap() });
        if (!splToken.value[0])
            return null;
        const balance = await rpc.getTokenAccountBalance(splToken.value[0].pubkey);
        return BigInt(balance.value.amount);
    }
    SolanaUtils.getBalance = getBalance;
    async function getBalances(chain, rpc, walletAddress, tokens) {
        let native;
        if (tokens.includes('native')) {
            native = BigInt(await rpc.getBalance(new web3_js_1.PublicKey(walletAddress)));
        }
        const splParsedTokenAccounts = await rpc.getParsedTokenAccountsByOwner(new web3_js_1.PublicKey(walletAddress), {
            programId: new web3_js_1.PublicKey(spl_token_1.TOKEN_PROGRAM_ID),
        });
        const balancesArr = tokens.map((token) => {
            if (token === 'native') {
                return { ['native']: native };
            }
            const addrString = new address_1.SolanaAddress(token).toString();
            const amount = splParsedTokenAccounts.value.find((v) => v?.account.data.parsed?.info?.mint === token)?.account.data.parsed?.info?.tokenAmount?.amount;
            if (!amount)
                return { [addrString]: null };
            return { [addrString]: BigInt(amount) };
        });
        return balancesArr.reduce((obj, item) => Object.assign(obj, item), {});
    }
    SolanaUtils.getBalances = getBalances;
    async function sendWait(chain, rpc, stxns) {
        const txhashes = await Promise.all(stxns.map((stxn) => rpc.sendRawTransaction(stxn)));
        const { blockhash, lastValidBlockHeight } = await rpc.getLatestBlockhash(rpc.commitment);
        await Promise.all(txhashes.map((txid) => {
            const bhs = {
                signature: txid,
                blockhash,
                lastValidBlockHeight,
            };
            return rpc.confirmTransaction(bhs, rpc.commitment);
        }));
        return txhashes;
    }
    SolanaUtils.sendWait = sendWait;
    async function getCurrentBlock(rpc) {
        return await rpc.getSlot(rpc.commitment);
    }
    SolanaUtils.getCurrentBlock = getCurrentBlock;
    function chainFromChainId(genesisHash) {
        const netChain = constants_1.solGenesisHashToNetworkChainPair.get(genesisHash);
        if (!netChain) {
            // TODO: this is required for tilt/ci since it gets a new genesis hash
            if (platform_1.SolanaPlatform.network === 'Devnet')
                return ['Devnet', 'Solana'];
            throw new Error(`No matching genesis hash to determine network and chain: ${genesisHash}`);
        }
        const [network, chain] = netChain;
        return [network, chain];
    }
    SolanaUtils.chainFromChainId = chainFromChainId;
    async function chainFromRpc(rpc) {
        const conn = rpc;
        const gh = await conn.getGenesisHash();
        return chainFromChainId(gh);
    }
    SolanaUtils.chainFromRpc = chainFromRpc;
})(SolanaUtils || (exports.SolanaUtils = SolanaUtils = {}));
//# sourceMappingURL=platformUtils.js.map